"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var environment_1 = require("@dapp-stack/environment");
var fs = require("fs-extra");
var path = require("path");
var sha1File = require("sha1-file");
var signale_1 = require("signale");
var parser = require("solidity-parser-antlr");
var signale = new signale_1.Signale({ scope: 'Doc' });
exports.runAll = function (contracts) {
    if (contracts === void 0) { contracts = false; }
    if (!contracts) {
        contracts = environment_1.build().compile.contracts;
    }
    contracts
        .filter(function (contract) { return contract.endsWith('.sol'); })
        .map(function (contract) { return path.join(environment_1.Structure.contracts.src, contract); })
        .forEach(function (contractFile) { return exports.run(contractFile); });
};
exports.run = function (contractFile) { return __awaiter(_this, void 0, void 0, function () {
    var outFile, filesTable, contractsTable, content, ast, reportContents;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                if (!fs.existsSync(contractFile)) {
                    signale.error("File not found: " + contractFile);
                    return [2 /*return*/];
                }
                outFile = path.join(environment_1.Structure.contracts.doc, path.basename(contractFile, '.sol') + ".md");
                return [4 /*yield*/, fs.ensureFile(outFile)];
            case 1:
                _a.sent();
                filesTable = "\n|  File Name  |  SHA-1 Hash  |\n|-------------|--------------|\n";
                contractsTable = "\n|  Contract  |         Type        |       Bases      |                  |                 |\n|:----------:|:-------------------:|:----------------:|:----------------:|:---------------:|\n|     \u2514      |  **Function Name**  |  **Visibility**  |  **Mutability**  |  **Modifiers**  |\n";
                filesTable += "| " + contractFile + " | " + sha1File(contractFile) + " |\n";
                content = fs.readFileSync(contractFile).toString('utf-8');
                ast = parser.parse(content, {});
                parser.visit(ast, {
                    ContractDefinition: function (node) {
                        var name = node.name;
                        var bases = node.baseContracts
                            .map(function (spec) {
                            return spec.baseName.namePath;
                        })
                            .join(', ');
                        var specs = '';
                        if (node.kind === 'library') {
                            specs += 'Library';
                        }
                        else if (node.kind === 'interface') {
                            specs += 'Interface';
                        }
                        else {
                            specs += 'Implementation';
                        }
                        contractsTable += "||||||\n| **" + name + "** | " + specs + " | " + bases + " |||\n";
                    },
                    FunctionDefinition: function (node) {
                        var name;
                        if (node.isConstructor) {
                            name = '\\<Constructor\\>';
                        }
                        else if (!node.name) {
                            name = '\\<Fallback\\>';
                        }
                        else {
                            name = node.name;
                        }
                        var spec = '';
                        if (node.visibility === 'public' || node.visibility === 'default') {
                            spec += 'Public ❗️';
                        }
                        else if (node.visibility === 'external') {
                            spec += 'External ❗️';
                        }
                        else if (node.visibility === 'private') {
                            spec += 'Private 🔐';
                        }
                        else if (node.visibility === 'internal') {
                            spec += 'Internal 🔒';
                        }
                        var payable = '';
                        if (node.stateMutability === 'payable') {
                            payable = '💵';
                        }
                        var mutating = '';
                        if (!node.stateMutability) {
                            mutating = '🛑';
                        }
                        contractsTable += "| \u2514 | " + name + " | " + spec + " | " + mutating + " " + payable + " |";
                    },
                    'FunctionDefinition:exit': function (node) {
                        contractsTable += " |\n";
                    },
                    ModifierInvocation: function (node) {
                        contractsTable += " " + node.name;
                    }
                });
                reportContents = "## Dapp's Description Report\n### Files Description Table\n" + filesTable + "\n### Contracts Description Table\n" + contractsTable + "\n### Legend\n|  Symbol  |  Meaning  |\n|:--------:|-----------|\n|    \uD83D\uDED1    | Function can modify state |\n|    \uD83D\uDCB5    | Function is payable |\n";
                try {
                    fs.writeFileSync(outFile, reportContents, { flag: 'w' });
                }
                catch (error) {
                    signale.error(error);
                }
                return [2 /*return*/];
        }
    });
}); };
//# sourceMappingURL=index.js.map